<?php namespace Maatwebsite\Excel\Classes;

use PhpOffice\PhpSpreadsheet\IOFactory;
use Illuminate\Filesystem\Filesystem;
use Maatwebsite\Excel\Exceptions\LaravelExcelException;

class FormatIdentifier
{
    const XLS = 'Xls';
    const XLSX = 'Xlsx';
    const ODS = 'Ods';
    const CSV = 'Csv';
    const HTML = 'Html';
    const XML = 'Xml';
    const SLK = 'Slk';
    const GNUMERIC = 'Gnumeric';
    const PDF = 'Pdf';

    const FORMATS = [
        self::XLS,
        self::XLSX,
        self::ODS,
        self::CSV,
        self::HTML,
        self::XML,
        self::SLK,
        self::GNUMERIC,
        self::PDF,
    ];

    const FORMAT_CONTENT_TYPES = [
        self::XLS => 'application/vnd.ms-excel; charset=UTF-8',
        self::XLSX => 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet; charset=UTF-8',
        self::CSV => 'application/csv; charset=UTF-8',
        self::HTML => 'HTML',
        self::PDF => 'application/pdf; charset=UTF-8',
    ];

    const DEFINED_FORMATS = [
        /*
        |--------------------------------------------------------------------------
        | Excel 2007
        |--------------------------------------------------------------------------
        */
        'xlsx' => self::XLSX,
        'xlsm' => self::XLSX,
        'xltx' => self::XLSX,
        'xltm' => self::XLSX,
        /*
        |--------------------------------------------------------------------------
        | Excel5
        |--------------------------------------------------------------------------
        */
        'xls' => self::XLS,
        'xlt' => self::XLS,
        /*
        |--------------------------------------------------------------------------
        | OOCalc
        |--------------------------------------------------------------------------
        */
        'ods' => self::ODS,
        'ots' => self::ODS,
        /*
        |--------------------------------------------------------------------------
        | SYLK
        |--------------------------------------------------------------------------
        */
        'slk' => self::SLK,
        /*
        |--------------------------------------------------------------------------
        | Excel2003XML
        |--------------------------------------------------------------------------
        */
        'xml' => self::XML,
        /*
        |--------------------------------------------------------------------------
        | Gnumeric
        |--------------------------------------------------------------------------
        */
        'gnumeric' => self::GNUMERIC,
        /*
        |--------------------------------------------------------------------------
        | HTML
        |--------------------------------------------------------------------------
        */
        'htm' => self::HTML,
        'html' => self::HTML,
        /*
        |--------------------------------------------------------------------------
        | CSV
        |--------------------------------------------------------------------------
        */
        'csv' => self::CSV,
        'txt' => self::CSV,
        /*
        |--------------------------------------------------------------------------
        | PDF
        |--------------------------------------------------------------------------
        */
        'pdf' => self::PDF,
    ];

//    /**
//     * Available formats
//     *
//     * @var array
//     * @access  protected
//     */
//    protected $formats = [
//        'Excel2007',
//        'Excel5',
//        'Excel2003XML',
//        'OOCalc',
//        'SYLK',
//        'Gnumeric',
//        'CSV',
//        'HTML',
//        'PDF'
//    ];

    /**
     * Construct new format identifier
     * @param Filesystem $filesystem
     */
    public function __construct(Filesystem $filesystem)
    {
        $this->filesystem = $filesystem;
    }

    /**
     * Get the file format by file
     * @param $file
     * @return string $format
     * @throws LaravelExcelException
     */
    public function getFormatByFile($file)
    {
        // get the file extension
        $ext = $this->getExtension($file);

        // get the file format
        $format = $this->getFormatByExtension($ext);

        // Check if the file can be read
        if ($this->canRead($file)) {
            return $format;
        }

        // Do a last try to init the file with all available readers
        return $this->lastResort($file, $format, $ext);
    }

    /**
     * Identify file format
     *
     * @param $ext
     * @return string|null
     */
    public function getFormatByExtension($ext): ?string
    {
        return self::DEFINED_FORMATS[$ext] ?? null;
    }

//
//    /**
//     * Identify file format
//     * @param $ext
//     * @return  string $format
//     */
//    public function getFormatByExtension($ext)
//    {
//        switch ($ext) {
//
//            /*
//            |--------------------------------------------------------------------------
//            | Excel 2007
//            |--------------------------------------------------------------------------
//            */
//            case 'xlsx':
//            case 'xlsm':
//            case 'xltx':
//            case 'xltm':
//                return 'Excel2007';
//                break;
//
//            /*
//            |--------------------------------------------------------------------------
//            | Excel5
//            |--------------------------------------------------------------------------
//            */
//            case 'xls':
//            case 'xlt':
//                return 'Excel5';
//                break;
//
//            /*
//            |--------------------------------------------------------------------------
//            | OOCalc
//            |--------------------------------------------------------------------------
//            */
//            case 'ods':
//            case 'ots':
//                return 'OOCalc';
//                break;
//
//            /*
//            |--------------------------------------------------------------------------
//            | SYLK
//            |--------------------------------------------------------------------------
//            */
//            case 'slk':
//                return 'SYLK';
//                break;
//
//            /*
//            |--------------------------------------------------------------------------
//            | Excel2003XML
//            |--------------------------------------------------------------------------
//            */
//            case 'xml':
//                return 'Excel2003XML';
//                break;
//
//            /*
//            |--------------------------------------------------------------------------
//            | Gnumeric
//            |--------------------------------------------------------------------------
//            */
//            case 'gnumeric':
//                return 'Gnumeric';
//                break;
//
//            /*
//            |--------------------------------------------------------------------------
//            | HTML
//            |--------------------------------------------------------------------------
//            */
//            case 'htm':
//            case 'html':
//                return 'HTML';
//                break;
//
//            /*
//            |--------------------------------------------------------------------------
//            | CSV
//            |--------------------------------------------------------------------------
//            */
//            case 'csv':
//            case 'txt':
//                return 'CSV';
//                break;
//
//            /*
//            |--------------------------------------------------------------------------
//            | PDF
//            |--------------------------------------------------------------------------
//            */
//            case 'pdf':
//                return 'PDF';
//                break;
//        }
//    }

    /**
     * Get the content type by file format
     * @param $format
     * @return string|null
     */
    public function getContentTypeByFormat($format): ?string
    {
        return self::FORMAT_CONTENT_TYPES[$format] ?? null;
    }

//    public function getContentTypeByFormat($format)
//    {
//        switch ($format) {
//
//            /*
//            |--------------------------------------------------------------------------
//            | Excel 2007
//            |--------------------------------------------------------------------------
//            */
//            case 'Excel2007':
//                return 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet; charset=UTF-8';
//                break;
//
//            /*
//            |--------------------------------------------------------------------------
//            | Excel5
//            |--------------------------------------------------------------------------
//            */
//            case 'Excel5':
//                return 'application/vnd.ms-excel; charset=UTF-8';
//                break;
//
//            /*
//            |--------------------------------------------------------------------------
//            | HTML
//            |--------------------------------------------------------------------------
//            */
//            case 'HTML':
//                return 'HTML';
//                break;
//
//            /*
//            |--------------------------------------------------------------------------
//            | CSV
//            |--------------------------------------------------------------------------
//            */
//            case 'CSV':
//                return 'application/csv; charset=UTF-8';
//                break;
//
//            /*
//            |--------------------------------------------------------------------------
//            | PDF
//            |--------------------------------------------------------------------------
//            */
//            case 'PDF':
//                return 'application/pdf; charset=UTF-8';
//                break;
//        }
//    }

    /**
     * Try every reader we have
     * @param        $file
     * @param bool $wrongFormat
     * @param string $ext
     * @return string $format
     * @throws LaravelExcelException
     */
    protected function lastResort($file, $wrongFormat = false, $ext = 'xls')
    {
        // Loop through all available formats
        foreach (self::FORMATS as $format) {
            if ($format === $wrongFormat) {
                continue;
            }

            // Check if the file could be read
            $reader = $this->initReaderByFileType($format);

            if ($reader && $reader->canRead($file)) {
                return $format;
            }
        }

        // Give up searching and throw an exception
        throw new LaravelExcelException('[ERROR] Reader could not identify file format for file [' . $file . '] with extension [' . $ext . ']');
    }

    /**
     * Check if we can read the file
     * @param $file
     * @return boolean
     */
    protected function tryRead($file)
    {
        $reader = $this->initReader($file);

        return $reader && $reader->canRead($file);
    }

    /**
     * Check if we can read the file
     * @param $file
     * @return boolean
     */
    protected function canRead($file)
    {
        $reader = $this->initReader($file);

        return $reader && $reader->canRead($file);
    }

//    protected function canRead($format, $file)
//    {
//        if ($format) {
//            $reader = $this->initReader($file);
//
//            return $reader && $reader->canRead($file);
//        }
//
//        return false;
//    }

    /**
     * Init the reader based on the format
     * @param string $format
     * @return \PhpOffice\PhpSpreadsheet\Reader\IReader
     */
    protected function initReaderByFileType($fileType)
    {
        // Create a new Reader of the type that has been identified
        return IOFactory::createReader($fileType);
    }

    /**
     * Init the reader based on the format
     * @param string $file
     * @return \PhpOffice\PhpSpreadsheet\Reader\IReader
     */
    protected function initReader($file)
    {
        //Identify the type of $inputFileName
        $inputFileType = IOFactory::identify($file);

        // Create a new Reader of the type that has been identified
        return IOFactory::createReader($inputFileType);
    }

//
//    /**
//     * Init the reader based on the format
//     * @param string $format
//     * @return \PhpOffice\PhpSpreadsheet\Reader\IReader
//     */
//    protected function initReader($format)
//    {
//        return IOFactory::createReader($format);
//    }

    /**
     * Get the file extension
     * @param string $file
     * @return string
     */
    protected function getExtension($file)
    {
        return strtolower($this->filesystem->extension($file));
    }
}
